@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-
+# -*- coding: utf-8 -*-
 """
-     Copyright (C) 2002-2004 Stas Z. <stas@linux.isbeter.nl>
+     Copyright (C) 2002-2006 Stas Z. <stas@linux.isbeter.nl>
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -63,7 +63,11 @@ except ImportError,info:
 ## Global options, config dic. Look in def get_rc() for format ##
 #__name__ = just to be consistent with a real config file
 #fullscr = 1 #set to toggle fullscreen
-rcdic_defaults = {'default':{'__name__':'default','fullscr':1,'language':'system'}}
+rcdic_defaults = {'default':{'__name__':'default',\
+                            'fullscr':1,\
+                            'language':'system',\
+                            'exclude_games':'',\
+                            'letter_size':'normal'}}
 
 if CPDEBUG: print "sys.argv",sys.argv
 if len(sys.argv) > 1:
@@ -77,6 +81,9 @@ if len(sys.argv) > 1:
         elif item == '--version':
             print version.version
             raise SystemExit
+        elif item == '--VERSION':
+            print version.VERSION
+            raise SystemExit
         elif item == '--help':
             print helptext
             raise SystemExit
@@ -177,14 +184,19 @@ def get_rc(file):
     return rcdic
     
 def set_globalrc(dic):
-    global rcdic,fullscr,LANG
+    """Here we check and make sure the rcdic is valid and also set some
+    global constants to get easy access to the rc values"""
+    global rcdic,fullscr,LANG,EXCLUDE_GAMES
+    #print "rcdic",rcdic
     try:
         fullscr = int(rcdic['default']['fullscr'])
         LANG = rcdic['default']['language']
+        EXCLUDE_GAMES = rcdic['default']['exclude_games']
     except (KeyError,ValueError,TypeError),info:
         print >> sys.stderr," Problems reading the childsplay config file,\n",\
-                info,"\n using the defaults"
+                info,"\n using the defaults and replacing the corrupt rc file"
         exc = sys.exc_info()[0]
+        print exc
         if exc is exceptions.KeyError:
             replace_rcfile()
         rcdic = rcdic_defaults
@@ -198,16 +210,29 @@ else:
     rcdic = rcdic_defaults
 set_globalrc(rcdic)
 
-# i18n
-stdloc = set_locale(LANG)
+# i18n, put a lot of exception caches because locale stuff keeps crashing
+# on badly configured platforms. Mac OSX locale support is totally :-( 
 try:
-    ChildsplayGoodies.language = stdloc
+    STDLOC,LOCALE_RTL = set_locale(LANG)
 except Exception,info:
     print >> sys.stderr,info
     ChildsplayGoodies.language = 'en'
-    stdloc = 'en'
-if CPDEBUG: print "Childsplay locale set to %s" % stdloc
-
+    STDLOC,LOCALE_RTL = 'en',None
+try:
+    #ChildsplayGoodies.language = STDLOC
+    # Add support for the LANGUAGE env var.
+    ChildsplayGoodies.language = STDLOC.split('@')[0].split('.')[0].split('_')[0]
+except Exception,info:
+    print >> sys.stderr,info
+    ChildsplayGoodies.language = 'en'
+    STDLOC,LOCALE_RTL = 'en',None
+if LOCALE_RTL and pyfribidi and STDLOC in ['he','ar']:
+    ChildsplayGoodies.locale_rtl = True
+    ChildsplayGoodies.pyfribidi = pyfribidi
+    
+if CPDEBUG: 
+    print "Childsplay locale set to %s" % STDLOC
+    print "Bidirectional flag is set to %s" % LOCALE_RTL
 ## Overrule rc values, this sucks. This must be rewritten and cleaned up
 ## XXX we should get a nice and clean way to handle config files and commandline options
 if len(sys.argv) > 1:
@@ -220,13 +245,24 @@ if len(sys.argv) > 1:
 
 # check if we have menu icons for the found games
 modules = []
+# See set_globalrc. We test for entries there so when a user has a old config
+# file without exclude_games, it will be replaced.
+# Otherwise we break support for older versions.
+#excludegames = rcdic['default']['exclude_games']
+###### TODO packid is removed when running under RTL locale
+###### It crashes :-(
+#if LOCALE_RTL: print "The pacman game is disabled for your locale, it crashes :-("
 for name in fnmatch.filter(os.listdir(MODULES),'*.py'):
     if CPDEBUG: print >> sys.stderr, "plugin",name
     if os.path.exists(os.path.join(DATADIR,'icons',name[:-3]+'.icon.png')):
-        modules.append((name[:-3]))
+        if name[:-3] in EXCLUDE_GAMES: #or name[:-3] == 'packid' and LOCALE_RTL:
+            continue
+        else:
+            modules.append((name[:-3]))
     else:
         print >> sys.stderr,'Can\'t find menu icon for',name,'\nRemoving',name,'from modules list'
     if CPDEBUG: print "gamemodules",modules   
+    
 ## setup a dic; key=menu icon pos., value = (module name, icon name).
 ## This dic is passed to the CPMenu constructor, in the main function.
 menudic = {}
@@ -330,17 +366,14 @@ class Score:
      It uses a 'scoresheet' which is a dic with lists as values and the game names
       as keys and is stored on disk as a 'pickle object' to get a cheap encrytion."""
     def __init__(self):
-        self.hsc_ttf = os.path.join(DATADIR,'bullpen3.ttf')
-        self.sc_ttf = os.path.join(DATADIR,'bluebold.ttf')
+        self.hsc_ttf = os.path.join(DATADIR,'Domestic_Manners.ttf')
+        self.sc_ttf = os.path.join(DATADIR,'VeraSeBd.ttf')
         self.sc_fsize = 32
         self.sc_fcol = (7,249,23)
-        """self.bal_l_img = load_image(os.path.join(BASEDIR,'Data','balloonsl.png'),1)
-        self.bal_r_img = load_image(os.path.join(BASEDIR,'Data','balloonsr.png'),1)
-        """
+        
         self.bal_l_img = Assets_img.get_assets(('balloonsl.png',))# This way get_assets returns one object
         self.bal_r_img = Assets_img.get_assets(('balloonsr.png',))
         try:
-            #f = open(os.path.join(BASEDIR,'Data','score'),'r')
             f = open(SCOREFILE,'r')
             self.score_sheet = pickle.load(f)
             f.close()
@@ -375,7 +408,6 @@ class Score:
     def _fetch_score(self):
         """ Load the high score file, makes a default score sheet when the load fails."""
         try:
-            #f = open(os.path.join(BASEDIR,'Data','score'),'r')
             f = open(SCOREFILE,'r')
             score_sheet = pickle.load(f)
             f.close()
@@ -388,7 +420,6 @@ class Score:
     def _put_sheet(self):
         """ Saves the high-score to disk, prints errors when it fails but continue anyway"""
         try:
-            #f = open(os.path.join(BASEDIR,'Data','score'),'w')
             f = open(SCOREFILE,'w')
             pickle.dump(self.score_sheet,f)
             f.close()
@@ -397,7 +428,6 @@ class Score:
             print >> sys.stderr,'Error in writing score'
             MyError.name = 'Error in writing score'
             MyError.line = info
-            #raise MyError
     
     def get_score(self,name):
         """ First get the file from disk (_fetch_score), then get the
@@ -433,12 +463,16 @@ class Score:
     
     def _get_key(self):
         """ Basic edeting function, used in the 'give your name'"""
-        # REMINDER, change this in using the new editing function from utils.py
-        # BUT, that function doesn't render caracter for caracter.
+        
+        key_map = None # Used as the default keymapping, see function map_keys()
+        if LOCALE_RTL and STDLOC:
+            key_map = {'he':he_key_map,
+                        'ar':ar_key_map}[STDLOC]
+        if STDLOC == 'ru': key_map = ru_key_map
         event = pygame.event.poll()
         if event.type is KEYDOWN:
-            if 96<event.key<123 :
-                surf,spam = font2surf(event.unicode,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
+            if 65<event.key<123 :
+                surf,spam = font2surf(map_keys(key_map,event.unicode),self.sc_fsize,self.sc_fcol,self.hsc_ttf)
                 return surf,event.unicode
             elif event.key is K_BACKSPACE:
                 return None,'back'
@@ -449,6 +483,8 @@ class Score:
     def _high_score(self,name,score):
         """ This starts the whole high-score event, usses it's own eventloop and
          takes care of everything"""
+        # TODO Make the RTL thing work
+        print "highscore locale",LOCALE_RTL,STDLOC
         sheet = self.get_score(name)
         self.sdim.dim(darken_factor=255,color_filter=(0,0,0))
         temp_back = pygame.Surface((800,500))
@@ -458,60 +494,35 @@ class Score:
         pygame.display.update(self.screen.blit(self.bal_r_img,(665,0)))
         
         Assets_snd.get_assets(('Winner2.ogg',)).play(1)
-                
+        
+        max_x = self.screen.get_width()# used to make RTL stuff screen size independant
         txt = _("Congratulations,")
         surf,spam = font2surf(txt,48,self.sc_fcol,self.hsc_ttf)
-        x = 150
+        x = 250
+        if LOCALE_RTL: x = max_x-surf.get_width()-x
         y=20
         pygame.display.update(self.screen.blit(surf,(x,y)))
         
         txt = _("You are a winner")
         surf,spam = font2surf(txt,32,self.sc_fcol,self.hsc_ttf)
-        x = 200
+        x = 300
+        if LOCALE_RTL: x = max_x-surf.get_width()-x
         y=120
         pygame.display.update(self.screen.blit(surf,(x,y)))
         
         txt = str(score)+_(" Points !!")
         surf,spam = font2surf(txt,48,self.sc_fcol,self.hsc_ttf)
-        x = 200
+        x = 160
+        if LOCALE_RTL: x = max_x-surf.get_width()-x
         y=200
         pygame.display.update(self.screen.blit(surf,(x,y)))
-        
-        txt = _("Please give your name: ")
-        surf,spam = font2surf(txt,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
-        x = 20
-        y=350
-        pygame.display.update(self.screen.blit(surf,(x,y)))
-        x += surf.get_width()
-        offset = 0
-        text = ''
-        surf = None
-        temp_back.blit(self.screen,(0,0),(0,0,800,500))
-        clock = pygame.time.Clock() 
-        while len(text) < 10:
-            clock.tick(30)
-            surf, key = self._get_key()
-            if not surf and not key: continue
-            #print key
-            if key != 'end' and key != 'back':
-                old_pos = self.screen.blit(surf,(x+offset,y))
-                pygame.display.update(old_pos)
-                offset += surf.get_width()
-                text = text+key
-            elif key == 'back' and len(text) > 0:
-                text = text[:-1]
-                if len(text) != 0:
-                    surf,spam = font2surf(text,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
-                    offset = surf.get_width()
-                else:
-                    offset = 0
-                    pygame.display.update(self.screen.blit(temp_back,(x,y),(x,y-4,300,100)))
-                    continue
-                #print x,y,temp_back
-                pygame.display.update(self.screen.blit(temp_back,(x,y),(x,y-4,300,100)))
-                pygame.display.update(self.screen.blit(surf,(x,y)))
-            elif key == 'end':
-                break
+
+        # This is the new RTL aware entry box
+        entry = EntryEdit(self,self.screen,(80,400),\
+                        _("Please give your name: "),\
+                        loc=(LOCALE_RTL,STDLOC))
+        text = entry.ask()
+
         #print text
         sheet.append(((score,text)))
         sheet.sort()
@@ -549,6 +560,7 @@ class Contr(Score):
         y = 600
         self.y = y
         self.x = x
+        #print "setup fullscreen>",fullscr
         if fullscr:
             self.screen = pygame.display.set_mode((x, y),FULLSCREEN)
         else:
@@ -561,7 +573,7 @@ class Contr(Score):
         except UnicodeEncodeError,info:
             print >> sys.stderr, info
         self.backcol = (80,151,244)
-        self.ttf = os.path.join(DATADIR,'bluehigh.ttf')
+        self.ttf = os.path.join(DATADIR,'Domestic_Manners.ttf')
         self.backgr = pygame.Surface((x,y))
         
         files = ('land.png','stop.png','question1.png','hscore.png','tux.png',\
@@ -571,7 +583,6 @@ class Contr(Score):
         
         Assets_img.set_transparent_image(0)
         Img.blue = Assets_img.get_assets(('blue.jpg',)) 
-        #Img.land = Assets_img.get_assets(('land.png',))
         Assets_img.set_transparent_image(1)
         
         self.backgr.blit(Img.blue, (0,0))
@@ -591,7 +602,11 @@ class Contr(Score):
                 
         Img.black = pygame.Surface(Img.firework1.get_size()).convert()
         
-        self.scorething = ScoreThing(24,(0,255,0))
+        if LOCALE_RTL:
+            ST_size = 16
+        else:
+            ST_size = 24
+        self.scorething = ScoreThing(ST_size,(0,255,0))
         self.scorething_place = (600,y-60)
         self.backgr.blit(self.scorething.update(),self.scorething_place)
         
@@ -650,7 +665,7 @@ class Contr(Score):
                         v = item.update(event)
                         if v:
                             if v == 'questionmark':
-                                stop = self.get_help(obj)
+                                stop = self.get_help(obj,18)
                             elif v == 'score' and obj.__class__.__name__ == 'Game':
                                 stop = self.show_score(Misc.helptitle)
                             elif v == 'stop':
@@ -677,8 +692,13 @@ class Contr(Score):
         fcol = (255,0,0)
         #fcol = (224,109,67) # Used for testing
         for line in text:
-            surf,size = font2surf(line,56,fcol,self.ttf)
-            dirtyrects.append((self.screen.blit(surf,(x,y))))
+            surf,size = font2surf(line,48,fcol,self.ttf)
+            if LOCALE_RTL:
+                l = surf.get_width()
+                dirtyrects.append((self.screen.blit(\
+                                    surf,(self.screen.get_width()-l-x,y))))
+            else:
+                dirtyrects.append((self.screen.blit(surf,(x,y))))
             y += surf.get_height() + 50
         pygame.display.update(dirtyrects)
         clock = pygame.time.Clock() 
@@ -690,7 +710,7 @@ class Contr(Score):
                 if event.type is KEYDOWN and event.key == ord(_("y").lower()):
                     self.sdim.dim(darken_factor=255,color_filter=(0,0,0))
                     surf,size = font2surf(_("Shutting down....."),72,(207,55,48),self.ttf)
-                    Snd.music1.stop()
+                    Snd.mainmusic.stop()
                     pygame.display.update(self.screen.blit(surf,(100,200)))
                     threads = threading.enumerate()
                     if len(threads) > 1:
@@ -769,27 +789,27 @@ class Contr(Score):
     def get_help(self,obj,fsize=26):
         """ Display the help text for the current game/plugin. It's using it's own eventloop.
           The text must be provided by the plugin."""
-        
+        if CPDEBUG: print "Starting get_help"
         if not obj.help():
             return 0
         ## cols and maxlines must be changed in conjuction with the fontsize 
-        cols = 86
-        maxlines = 13
+        cols = 74
+        maxlines = 15
         
         helptxt = txtfmt(obj.help(), cols)
         
         helptitle = obj.helptitle()
         oldclip = self.screen.get_clip()
         self.screen.set_clip()
-                
+        ttf = os.path.join(DATADIR,'VeraSeBd.ttf')
         lines = len(helptxt)
         if CPDEBUG: print "get_help lines",lines
         for i in range(0, lines, maxlines):
             self.sdim.undim()
-            self.sdim.dim(darken_factor=180,color_filter=(51,10,10))
+            self.sdim.dim(darken_factor=200,color_filter=(51,10,10))
             update_rects = []
             part = helptxt[i:i+maxlines]
-            surf = txt2surf(part,self.x-20,self.y-104,fsize,helptitle,1,(158,196,107),self.ttf)
+            surf = txt2surf(part,self.x-20,self.y-104,fsize,helptitle,1,(158,196,107),ttf)
             update_rects.append((self.screen.blit(surf,(0,0))))
             if i+maxlines < lines: #more to come
                 okrect = self.screen.blit(Img.OK_but,(self.x-700,self.y-90))
@@ -833,12 +853,18 @@ class Contr(Score):
         gameoverobj = []
         x = 20
         for item in 'GAME OVER':# can be any text
-            letter,size = font2surf(item,96,(209,249,9),self.ttf)
+            try:
+                font = pygame.font.Font(self.hsc_ttf,124)
+            except Exception,info:
+                print >> sys.stderr,info,'\nUsing standard pygame font'
+                font = pygame.font.Font(None,96)
+            surf = font.render(item, 1, (209,249,9))
             if item == " ": # we don't use spaces
-                x = x + size[0][1]-16
+                x = x + 48
                 continue
-            gameoverobj.append((Firework(x,letter)))
-            x = x + size[0][1]-16
+            gameoverobj.append((Firework(x,surf)))
+            #x = x + size[0][1]-48
+            x = x + surf.get_width()
         
         self.sdim.dim(darken_factor=255,color_filter=(0,0,0))
         for obj in gameoverobj:
@@ -861,7 +887,10 @@ def main():
     global screendim,game_score
     
     c  = Contr()# Start master control
-    Assets_snd.get_assets('music1.ogg', Snd)
+    Assets_snd.get_assets('music1.ogg', Snd)# will become Snd.music1
+    Snd.galaga = load_sound(os.path.join(DATADIR,'Galaga1.ogg'))
+    Snd.dub = load_sound(os.path.join(DATADIR,'dub.ogg'))
+    music_collection = [Snd.music1,Snd.galaga,Snd.dub]
     #modul = 'Menu'
     screendim = Dimmer(keepalive=1) # dimmer is used for screen copying
     screendim.dim(darken_factor=0) # just stores a copy in Dimmer.buffer
@@ -870,14 +899,15 @@ def main():
     menu = CPMenu(screen,back,menudic,BASEDIR)
     
     while 1:
-        Snd.music1.play(-1)
+        Snd.mainmusic = random.choice(music_collection)
+        Snd.mainmusic.play(-1)
         menu.start()
         plugin_name,spam = c.mainloop(menu)
         if CPDEBUG: print >> sys.stderr,"menu choice",plugin_name   
         c.screen.blit(screendim.buffer,(0,0))
         c.backgr.blit(screendim.buffer,(0,0))
         screendim.undim()
-        Snd.music1.stop()
+        Snd.mainmusic.stop()
         stop = None
         try:
             # import game module
