@@ -1,4 +1,4 @@
-
+# -*- coding: utf-8 -*-
 """
      Copyright (C) 2002-2004 Stas Z <stas@linux.isbeter.nl>
 
@@ -20,10 +20,10 @@
  See the childsplay reference for more information
 """
 import pygame,os,sys,fnmatch,traceback,operator,imp,glob,locale,\
-        gettext,__builtin__,shutil
-from pygame.constants import RLEACCEL,KEYDOWN,K_BACKSPACE,K_RETURN,\
-                                MOUSEBUTTONDOWN,K_ESCAPE
+        gettext,__builtin__,shutil,types,string
+from pygame.constants import *
 from CPConstants import *
+
 DEBUG = 0
 
 class MyError(Exception):
@@ -32,41 +32,136 @@ class MyError(Exception):
     extra = None
 
 def set_locale(lang=None):
+    global LOCALE_RTL,pyfribidi
     txt = ""
-    print "Setting childsplay locale to ",lang
     try:
         if not lang or lang == 'system':
             try:
-                lang = locale.getdefaultlocale()[0].split("_")[0]
-            except ValueError:
-                print >> sys.stderr,info
+                # FIX locale.py LANGUAGE parsing bug, the fix was added on the
+                # upstream CVS on the 1.28.4.2 revision of 'locale.py', It
+                # should be included on Python 2.4.2.
+                if os.environ.has_key('LANGUAGE'):
+                    lang = os.environ['LANGUAGE'].split(':')[0]
+                else:
+                    lang = locale.getdefaultlocale()[0]
+            except ValueError, info:
+                print >> sys.stderr, info
                 lang = 'en'
-        if lang == "en": 
-            __builtin__.__dict__['_'] = lambda x:x
-            txt = "Language set to %s" % lang
-            return 'en'
+        languages = [ lang ]
+        if os.environ.has_key('LANGUAGE'):
+          languages += os.environ['LANGUAGE'].split(':')
+        print "Setting childsplay locale to '%s'" % (lang)
         lang_trans = gettext.translation('childsplay',\
                                      localedir=LOCALEDIR,\
-                                     languages=[lang])
+                                     languages=languages)
         __builtin__.__dict__['_'] = lang_trans.ugettext
-    except Exception,info:
-        txt = "Cannot set language to %s \n switching to English" % lang
-        print >> sys.stderr,info,txt
+    except Exception, info:
+        txt=""
+        if lang and lang.split('@')[0].split('.')[0].split('_')[0] != 'en':
+            txt = "Cannot set language to '%s' \n switching to English" % lang
+            print >> sys.stderr,info,txt
         __builtin__.__dict__['_'] = lambda x:x
         lang = 'en'
+    else:
+        lang = lang.split('@')[0].split('.')[0].split('_')[0]
+    # This is to signal that we running under a RTL locale like Hebrew or Arabic
+    # Only Hebrew is supported until now
+    if lang == 'he':
+        if pyfribidi:
+            LOCALE_RTL = True
+        else:
+            print "Your locale needs RTL support, but I can't find pyfribidi."
+            print "See http://hspell-gui.sourceforge.net/pyfribidi.html."
+            print "Cannot set language to '%s' \n switching to English" % lang
+            lang = 'en'
+        
+    return (lang,LOCALE_RTL)
+    
+def get_locale():
+    """Get the systems locale.
+    This can be different from the locale used by childsplay.(see set_locale())"""
+    try:
+        lang = ''
+        # FIX locale.py LANGUAGE parsing bug, the fix was added on the
+        # upstream CVS on the 1.28.4.2 revision of 'locale.py', It
+        # should be included on Python 2.4.2.
+        if os.environ.has_key('LANGUAGE'):
+            lang = os.environ['LANGUAGE'].split(':')[0]
+        # This makes sure that we never return a value of None.
+        # This is a fix for systems that set LANGUAGE to ''.
+        if lang == '':
+            lang = locale.getdefaultlocale()[0]
+    except Exception,info:
+        print >> sys.stderr,info
+        print "Switching to English"
+        lang = 'en'
+    if lang == 'C' or lang.lower() == 'posix':
+        # Default to English
+        lang = 'en'
     return lang
     
-def get_locale():
-    """Return the current locale setting."""
+## Work in progress, to be used in conjuction with a RTL language
+def _check_LocaleIsRTL(seq):
+    """This will check if we should use different stuff for the locale.
+    Returns -> tuple: ttf font to use, reversed seq.
+    """
+    rtl_ttf,rtl_seq = None,seq
+    # Only Hebrew is supported at this moment
     try:
-        loc = locale.getdefaultlocale()[0]
-    except ValueError:
-        print >> sys.stderr,info
-        loc = 'en'
-    if loc == 'C' or loc == 'posix':
-        # Default to English
-        loc = 'en'
-    return loc
+        if  get_locale()[:2].split('@')[0].split('.')[0].split('_')[0] != "he":
+            return (None,seq)
+    except Exception,info:
+        print info
+        print "Error checking locale setting"
+        print "Make sure your locale is configured properly"
+        return (None,seq)
+    if type(seq) is types.ListType:
+        rtl_seq = []
+        for line in seq:
+            if DEBUG:
+                print "from gettext",line.encode('utf-8')
+                print "from pyfribidi",pyfribidi.log2vis(line.encode('utf-8'))
+            rtl_seq.append(unicode(pyfribidi.log2vis(line.encode('utf-8')),'utf-8'))
+    else:
+        if DEBUG:
+            print "from gettext",seq.encode('utf-8')
+            print "from pyfribidi",pyfribidi.log2vis(seq.encode('utf-8'))
+        rtl_seq = unicode(pyfribidi.log2vis(seq.encode('utf-8')),'utf-8')
+    if os.path.exists('/usr/share/fonts/truetype/freefont/FreeSans.ttf'):
+        rtl_ttf = '/usr/share/fonts/truetype/freefont/FreeSans.ttf'
+    elif os.path.exists('/usr/local/share/fonts/truetype/freefont/FreeSans.ttf'):
+        rtl_ttf = '/usr/local/share/fonts/truetype/freefont/FreeSans.ttf'
+    elif os.path.exists(os.path.join(DATADIR,'he.ttf')):
+        rtl_ttf = os.path.join(DATADIR,'he.ttf')
+    else:
+        print "No suitable fonset found for your locale"
+        rtl_ttf = None
+    if DEBUG: print "using fontset %s \nfor RTL locale" % rtl_ttf
+    return (rtl_ttf,rtl_seq)
+
+def read_unicode_file(path):
+    import codecs
+    file = []
+    try:
+        f = codecs.open(path, 'r',encoding='utf-8')
+##        for line in f:
+##            file.append(repr(line))
+        file = f.readlines()
+    except Exception,info:
+        return None
+    return file
+
+def map_keys(key_map,key):
+    """map_keys --> mapped key when a map is specified else the key is returnt
+    unchanged.
+    @key_map is the keymap to use or None when no mapping is needed.
+    @key is the key to map.
+    Used to provide a keymapping for Hebrew, Arabic and Russian."""
+    if not key_map:
+        return key
+    if key in dict.keys(key_map):
+        return key_map[key]
+    return key
 
 def replace_rcfile():
     """This is only used when we want to replace an existing configfile."""
@@ -126,8 +221,8 @@ def load_music(file):
             pygame.mixer.music.play(loop)
         def stop(self):
             pygame.mixer.music.stop()
-	def queue(self):
-	    pygame.mixer.music.queue(self.s)
+    def queue(self):
+        pygame.mixer.music.queue(self.s)
     
     class NoneSound:
             def play(self,loop=0):
@@ -155,22 +250,28 @@ def load_image(file, transparent=0, alpha=0):
     if transparent :
         surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
     if alpha:
-	if DEBUG: print "pyassetmlSDL alpha image",file
-	#if the image loaded has alpha, the overall alpha set here
-	#(128) will be overriden. Do it anyway for forcing RLEACCEL
-	surface.set_alpha(128, RLEACCEL)
-	# if the display is in a lower mode, then surface.convert would
-	# convert the surface in an alpha-less surface.
-	return surface
-
+        if DEBUG: print "pyassetmlSDL alpha image",file
+        #if the image loaded has alpha, the overall alpha set here
+        #(128) will be overriden. Do it anyway for forcing RLEACCEL
+        surface.set_alpha(128, RLEACCEL)
+        # if the display is in a lower mode, then surface.convert would
+        # convert the surface in an alpha-less surface.
+        return surface
     return surface.convert()
 
 def txt2surf(seq,x,y,fsize=18,title=None,box=None,fcol=(0,0,0),ttf=None):
     """Render a list of strings in a surface with the screen backgr -> pygame surface"""
     space = 16 #space at the left for the text
     offset = 8 # and top
+    
     helpsurf = pygame.Surface((x,y)).convert()
     helpsurf.blit(pygame.display.get_surface(),(0,0))
+    ## Work in progress. 
+    # This check must be done everytime because users can change locale on the fly.
+    rtl_ttf,rtl_seq = _check_LocaleIsRTL(seq)
+    if rtl_ttf:
+        ttf = rtl_ttf
+        seq = rtl_seq
     if box:
         r = pygame.draw.rect(helpsurf,(255,255,255),(8,8,x-16,y-16),2)
         helpsurf.set_clip(r.inflate(-16,-16))
@@ -180,11 +281,16 @@ def txt2surf(seq,x,y,fsize=18,title=None,box=None,fcol=(0,0,0),ttf=None):
         offset += surf.get_height()+ 8
     try:
         font = pygame.font.Font(ttf,fsize)
-    except RuntimeError,info:
+    except Exception,info:
         print >> sys.stderr,info,'\nUsing standard pygame font'
         font = pygame.font.Font(None,fsize)
     for line in seq:
-        helpsurf.blit(font.render(line, 1, fcol),(space, offset))
+        fsurf = font.render(line, 1, fcol)
+        if rtl_ttf:
+            l = fsurf.get_width()
+            helpsurf.blit(fsurf,(x-l-space, offset))
+        else:
+            helpsurf.blit(fsurf,(space, offset))
         offset += fsize + 8
     return helpsurf
 
@@ -193,6 +299,11 @@ def char2surf(char,size,fcol=None,ttf=None):
     char must be a sting. Returns the surface."""
     if not fcol:
         fcol = (0,0,0)
+    # This check must be done everytime because users can change locale on the fly.
+    rtl_ttf,rtl_char = _check_LocaleIsRTL(char)
+    if rtl_ttf:
+        ttf = rtl_ttf
+        char = rtl_char
     try:
         font = pygame.font.Font(ttf,size)
     except RuntimeError,info:
@@ -205,12 +316,18 @@ def char2surf(char,size,fcol=None,ttf=None):
     return s
 
 def font2surf(word,size,fcol=None,ttf=None,sizel=None):
-    """Renders a text in a surface and returns a surface,size list of the items"""
+    """Renders a text in a surface and returns a surface,size list of the items
+    sizelist is a list like this [(7,17),(10,17)] tuples are x,y size of the character."""
     if not fcol:
         fcol = (0,0,0)
+    # This check must be done everytime because users can change locale on the fly.
+    rtl_ttf,rtl_word = _check_LocaleIsRTL(word)
+    if rtl_ttf:
+        ttf = rtl_ttf
+        word = rtl_word
     try:
         font = pygame.font.Font(ttf,size)
-    except RuntimeError,info:
+    except Exception,info:
         print >> sys.stderr,info,'\nUsing standard pygame font'
         font = pygame.font.Font(None,size)
     if sizel:
@@ -388,45 +505,87 @@ def import_module(filename, globals=None, locals=None, fromlist=None):
         trace_error()
         raise MyError,text
         if fp: fp.close()
-                
-def entry_edit(entry="",maxlen=10,but_ok_rect=(0,0,0,0),but_cancel_rect=(0,0,0,0)):
-    """ Basic entry field editing function. Returns a updated entry string.
-      Starts his own event loop, and only breaks when some user event occurs.
-      entry - string with the entry text ("" in the beginning)
-      but_ok_rect - pygame rect from the (optional) ok-button.
-      but_cancel_rect - same as but_ok for the cancel button.
-      """
-    stop = 0
-    but_ok_rect = pygame.Rect(but_ok_rect)
-    but_cancel_rect = pygame.Rect(but_ok_rect)
-    while stop == 0:
-        stop = 1 # we stop when we hit a 'break' else set stop to 0
-        for event in pygame.event.get():
-            if event.type is KEYDOWN:
-                if 31 < event.key < 123 and len(entry) < maxlen: # printable chr
-                    entry = "%s%c" % (entry,event.key)
-                    break
-                elif event.key is K_BACKSPACE and entry:
-                    entry = entry[:-1]
-                    break
-                elif event.key is K_RETURN:
-                    break
-                elif event.key is K_ESCAPE:
-                    entry = ""
-                    break
-                
-            elif event.type is MOUSEBUTTONDOWN:
-                pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
-                if but_ok_rect.contains(pos):
-                    break
-                elif but_cancel_rect.contains(pos):
-                    entry = ""
-                    break
-        else:
-            stop = 0 # we don't hit break so we loop
-        pygame.event.pump()
-    return entry            
+
+class EntryEdit:
+    """This class must be implemented by the childsplay Score class.
+    Class provides a RTL aware'entry edit' object which renders the users
+    input onto the screen. This screen is updated by this class with a call to
+    pygame.display.update(the changed rect)
+    The font, size and color is taken from the parent class.
+    """
     
+    def __init__(self,parent,screen,pos,prompt,loc):
+        self.locale_rtl,self.stdloc = loc
+        self.surf=screen
+        self.prompt=prompt
+        self.parent = parent
+        self.sc_fsize = self.parent.sc_fsize
+        self.sc_fcol = self.parent.sc_fcol
+        self.hsc_ttf = self.parent.hsc_ttf
+        ### RTL mapping related ######
+        self.key_map = None # Used as the default keymapping, see function utils.map_keys()
+        if DEBUG: print "self.locale_rtl",self.locale_rtl,"stdloc",self.stdloc
+        if self.locale_rtl and self.stdloc:
+            self.key_map = {'he':he_key_map,
+                        'ar':ar_key_map}[self.stdloc]
+        if self.stdloc == 'ru': self.key_map = ru_key_map
+        ### end ####
+        # Prevent from loops from using 100% CPU
+        self.clock = pygame.time.Clock()
+        self.pos = pos
+        self.maxlen = 16
+        # calaulate maximun length of the enty box
+        # "t" * maxlen seems to give a nice average
+        #s,spam = font2surf(prompt+"t"*self.maxlen,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
+        self.rect = pygame.Rect(pos[0],pos[1],700,100)
+        
+    def _get_key(self):
+        while 1:
+            self.clock.tick(30)
+            event = pygame.event.poll()
+            if event.type == KEYDOWN:
+                m = pygame.key.get_mods()
+                # when using key_maps : < > and ? need spatial care (keys 59,47,46,44)
+                if self.locale_rtl and event.key == 59 or event.key == 47 or \
+                            event.key == 46 or event.key == 44:
+                    return event.key
+                if m & KMOD_SHIFT:
+                    return event.key-32
+                return event.key
+            else:
+                pass
+                
+    def ask(self):
+        """ask() -> answer
+        This will renders the question and answer onto the screen and return
+        the answer when the user hits enter.
+        The anwser can be max 16 characters"""
+                
+        current_string = []
+        self.display_box(self.prompt + string.join(current_string,""))
+        while 1:
+            inkey = self._get_key()
+            if inkey == K_BACKSPACE:
+                current_string = current_string[0:-1]
+            elif inkey == K_RETURN:
+                break        
+            elif len(current_string) < self.maxlen and 31 < inkey < 127:
+                current_string.append(map_keys(self.key_map,chr(inkey)))
+            self.display_box(self.prompt+string.join(current_string,""))
+        return string.join(current_string,"")
+    
+    def display_box(self,message):
+        """Print a message on the screen"""
+        pygame.draw.rect(self.surf, (0,0,0),self.rect, 0)
+        ## For testing only
+        #pygame.draw.rect(self.surf, (255,255,255),self.rect, 1)
+        if len(message) != 0:
+            s,spam = font2surf(message,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
+            if self.locale_rtl:
+                self.surf.blit(s,(self.rect[2]-s.get_width(),self.rect[1]+4))
+            else:
+                self.surf.blit(s,(self.rect[0]+4,self.rect[1]+4))
+        pygame.display.update()
 
 class ReturnObject:
     """ Class object to be used as the return object in the plugins eventloop.
@@ -448,81 +607,7 @@ class ReturnObject:
         ###### NOT IN USE YET, this is for the monitor plugin (Very alpha)#####
         ##self.xml_dic = {'col0':[], 'col1':[], 'col2':[], 'col3':[], 'pre':[]}
         
-class Blink:
-    """ A general class object to provide a 'blinking' image.
-      Blink(img,img_h,pos,*args)
-      img = image 1
-      img_h = image 2 (the highlight version)
-      pos = position to blit, tuple(x,y)
-      *args = optional data gets returned in the DATA attribute
-      The DATA attribute gets set by the instance which is selected
-      by the mouse position.
-    """
-    DATA = None
-    def __init__(self,img,img_h,pos,*args):
-        self.img = img
-        self.img_h = img_h
-        self.image = self.img
-        self.rect = self.image.get_rect().move(pos)
-        self.data = args
-        
-    def update(self):
-        """ Update the blink object -> the tuple (switched image,rect,data)""" 
-        if self.rect.inflate(-10,-10).contains((pygame.mouse.get_pos()+(1,1))):
-            self.image = self.img_h
-            Blink.DATA = self.data
-            return self.image,self.rect,self.data
-        elif Blink.DATA is self.data:# Not selected by the mouse
-            Blink.DATA = None # then unset DATA
-            
-        if self.image is self.img:
-            self.image = self.img_h
-        else:
-            self.image = self.img
-        return self.image,self.rect,None
 
-def pick_one(scr,rects,field=None):
-    """ pick_one(scr,rects,field=None) -> SDL surface.
-     scr = main SDL screen.
-     rects = list of displayed rects to choose from. 
-     field = pygame.Rect of the area containing the rects.
-     
-     The rects are 'highlighted' when the mouse goes over them, sort of 'mouse-over-javascript'.
-     When the user clicks a button the surface corresponding wiyh the current rect
-     is returnt"""
-    
-    if not field:
-        field = scr.get_rect()
-    if len(rects) < 2:
-        MyError.name = " Utils.pick_one: length of the rects list should be more than one.\n Not %s" % len(rects)
-        raise MyError
-        
-    oldhot_spot = None
-    iconsheet = pygame.display.get_surface().convert()# used as background to blit when the mouse moves
-    while 1: #begin icon picking
-        pygame.event.pump()
-        events = pygame.event.get()
-        for event in events:
-            if event.type is KEYDOWN and event.key is K_ESCAPE:
-                MyError.name = "Utils.pick_one(%s,%s,%s)" % (scr,rects,field)
-                raise MyError
-            pos = pygame.mouse.get_pos()+(4,4)
-            if field.contains(pos):
-                pos = pygame.Rect(pos)
-                index = pos.collidelist(rects)
-                if index == -1:# not found
-                    break
-                hot_spot = rects[index]
-                if not oldhot_spot:
-                    oldhot_spot = hot_spot
-                if hot_spot != oldhot_spot:
-                    scr.blit(iconsheet, oldhot_spot, oldhot_spot)
-                    pygame.draw.rect(scr, (234,14,32), hot_spot.inflate(-1,-1), 2)
-                    pygame.display.update((hot_spot, oldhot_spot))
-                    oldhot_spot = hot_spot
-                
-                if event.type is MOUSEBUTTONDOWN:
-                    return iconsheet.subsurface(hot_spot)
 
 
 def __string_split(line, split):
@@ -750,4 +835,7 @@ class ChildsplayGoodies:
     These references are put in here at runtime by the childsplay main module (childsplay.py)
     and can be used by the game modules to maniputlate some aspects of childsplay.
     The best way to query this object is to look at it from a game module."""
+    global pyfribidi
     language = "en"
+    locale_rtl=False
+    py_fribidi = pyfribidi
